// ugviewView.cpp : implementation of the CVtkView class
//

#include "stdafx.h"
//#include "ugview.h"

#include "vtkDoc.h"
#include "vtkView.h"
#include "vtkWin32OpenGLRenderWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVtkView

IMPLEMENT_DYNCREATE(CVtkView, CView)

BEGIN_MESSAGE_MAP(CVtkView, CView)
//{{AFX_MSG_MAP(CVtkView)
ON_WM_CREATE()
ON_WM_CHAR()
ON_WM_TIMER()
ON_WM_RBUTTONDOWN()
ON_WM_RBUTTONUP()
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
//}}AFX_MSG_MAP
// Standard printing commands
ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVtkView construction/destruction

CVtkView::CVtkView()
{
  // TODO: add construction code here
  iren = NULL;
  ren = NULL;
  renWindow = NULL;
  
  //CopySource = NULL;
  //CopyMedium = NULL;
  
  DIBScale = 1.0;
  DIBDepth = 24;
}

CVtkView::~CVtkView()
{
  if (iren!=NULL) 
    {
    iren->Delete();
    iren = NULL;
    ren->Delete();
    ren = NULL;
    renWindow->Delete();
    renWindow = NULL;
    }
}

BOOL CVtkView::PreCreateWindow(CREATESTRUCT& cs)
{
  // TODO: Modify the Window class or styles here by modifying
  //  the CREATESTRUCT cs
  cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN | CS_OWNDC;
  return CView::PreCreateWindow(cs);
}

BYTE *CVtkView::GetDIB()
{
  BYTE *val, *img;
  int *size, y, x, rowSize;
  unsigned char *rgbs, *rgbTmp;
  BITMAPINFOHEADER *header;

  size = this->renWindow->GetSize();
	
  // make sure all rows are long aligned
  rowSize = size[0]*3;
  rowSize = rowSize + ((4 - (rowSize%4))%4);

  val = (BYTE *)malloc(rowSize*size[1] + sizeof(BITMAPINFOHEADER));
  header = (BITMAPINFOHEADER *)val;
  header->biSize = sizeof(BITMAPINFOHEADER);
  header->biWidth = size[0];
  header->biHeight = size[1];
  header->biPlanes = 1;
  header->biBitCount = 24;
  header->biCompression = BI_RGB;
  header->biClrUsed = 0;
  header->biClrImportant = 0;
  header->biSizeImage = rowSize*size[1];
  img = val + sizeof(BITMAPINFOHEADER);

  rgbs = this->renWindow->GetPixelData(0,0,size[0]-1,size[1]-1,0);
  rgbTmp = rgbs;

  for (y = 0; y < size[1]; y++)
    {
    for (x = 0; x < size[0]; x++)
      {
      *img = rgbTmp[2]; img++;
      *img = rgbTmp[1]; img++;
      *img = rgbTmp[0]; img++;
      rgbTmp += 3;
      }
    img = img + (rowSize - size[0]*3);
    }
  delete [] rgbs;
  return val;
}

/////////////////////////////////////////////////////////////////////////////
// CVtkView drawing

void CVtkView::OnDraw(CDC* pDC)
{ 
  if (this->ren)
    {
    if (pDC->IsPrinting())
      {
      int *size = this->renWindow->GetSize();

      BeginWaitCursor();

      int cxDIB = size[0]*this->DIBScale;         // Size of DIB - x
      int cyDIB = size[1]*this->DIBScale;         // Size of DIB - y
      CRect rcDest;

      // get size of printer page (in pixels)
      int cxPage = pDC->GetDeviceCaps(HORZRES);
      int cyPage = pDC->GetDeviceCaps(VERTRES);
      // get printer pixels per inch
      int cxInch = pDC->GetDeviceCaps(LOGPIXELSX);
      int cyInch = pDC->GetDeviceCaps(LOGPIXELSY);

      //
      // Best Fit case -- create a rectangle which preserves
      // the DIB's aspect ratio, and fills the page horizontally.
      //
      // The formula in the "->bottom" field below calculates the Y
      // position of the printed bitmap, based on the size of the
      // bitmap, the width of the page, and the relative size of
      // a printed pixel (cyInch / cxInch).
      //
      rcDest.bottom = rcDest.left = 0;
      if (((float)cyDIB*(float)cxPage/(float)cxInch) > 
	        ((float)cxDIB*(float)cyPage/(float)cyInch))
	      {
        rcDest.top = cyPage;
	      rcDest.right = ((float)(cyPage*cxInch*cxDIB)) /
	        ((float)(cyInch*cyDIB));
	      }
      else
	      {
	      rcDest.right = cxPage;
	      rcDest.top = ((float)(cxPage*cyInch*cyDIB)) /
	         ((float)(cxInch*cxDIB));
	      } 

      CRect	rcDestLP(rcDest);
      pDC->DPtoLP(rcDestLP);
      iren->DoPalette(pDC->GetSafeHdc());
      pDC->SetStretchBltMode(COLORONCOLOR);

      iren->StretchDIB(pDC->GetSafeHdc(),rcDestLP.left,rcDestLP.bottom,
		       rcDestLP.right - rcDestLP.left, 
		       rcDestLP.top - rcDestLP.bottom,
		       size[0]*this->DIBScale,
		       size[1]*this->DIBScale,
		       this->DIBDepth);

      EndWaitCursor();
      }
    else
      {
      renWindow->Render();
      SwapBuffers(this->hDC);
      }
    }
}

/////////////////////////////////////////////////////////////////////////////
// CVtkView printing

BOOL CVtkView::OnPreparePrinting(CPrintInfo* pInfo)
{
  // default preparation
  pInfo->SetMinPage(1);
  pInfo->SetMaxPage(1);
  return DoPreparePrinting(pInfo);
}

void CVtkView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
  // TODO: add extra initialization before printing
}

void CVtkView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
  // TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CVtkView diagnostics

#ifdef _DEBUG
void CVtkView::AssertValid() const
{
  CView::AssertValid();
}

void CVtkView::Dump(CDumpContext& dc) const
{
  CView::Dump(dc);
}

CVtkDoc* CVtkView::GetDocument() // non-debug version is inline
{
  ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVtkDoc)));
  return (CVtkDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVtkView message handlers


int CVtkView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  if (CView::OnCreate(lpCreateStruct) == -1)
    return -1;
	
  // TODO: Add your specialized creation code here
  if (renWindow == NULL) 
    {
    renWindow = vtkRenderWindow::New();
    renWindow->SwapBuffersOn();
    iren = new vtkWin32MappedInteractor;
    iren->SetRenderWindow(renWindow);
	
    hWnd = GetSafeHwnd();
    hDC = ::GetDC(hWnd); // device context for painting
	
    CRect	rcBounds;
    this->GetClientRect(rcBounds);

    CRect	rcBoundsDP(rcBounds);
    this->GetDC()->LPtoDP(rcBoundsDP);
    ren = NULL;
    }
		
  return 0;
}


void CVtkView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnChar(this->m_hWnd,nChar, nRepCnt, nFlags);
}

void CVtkView::OnTimer(UINT nIDEvent) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnTimer(this->m_hWnd,nIDEvent);
}

void CVtkView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnRButtonDown(this->m_hWnd,nFlags, point);	
}

void CVtkView::OnRButtonUp(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnRButtonUp(this->m_hWnd,nFlags, point);	
}

void CVtkView::OnLButtonDown(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnLButtonDown(this->m_hWnd,nFlags, point);
}

void CVtkView::OnLButtonUp(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) iren->OnLButtonUp(this->m_hWnd,nFlags, point);
}

void CVtkView::OnMouseMove(UINT nFlags, CPoint point) 
{
  if (ren) iren->OnMouseMove(this->m_hWnd,nFlags, point);
}

void CVtkView::OnSize(UINT nType, int cx, int cy) 
{
  if (ren) iren->OnSize(this->m_hWnd,nType, cx, cy);
}	

BOOL CVtkView::OnEraseBkgnd(CDC* pDC) 
{
  // TODO: Add your message handler code here and/or call default
  if (ren) return TRUE;
  return CView::OnEraseBkgnd(pDC);
}

void CVtkView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
  // TODO: Add your specialized code here and/or call the base class
  // create a renderer if necc
  if (ren==NULL && this->GetDocument()->m_actor) 
    {
    CRect rcBounds;
    this->GetClientRect(rcBounds);
    CRect rcBoundsDP(rcBounds);
    this->GetDC()->LPtoDP(rcBoundsDP);
	
    this->iren->Initialize(this->GetSafeHwnd(),&rcBoundsDP,renWindow);
    this->ren = vtkRenderer::New();
    this->renWindow->AddRenderer(this->ren);
    }

  if (this->ren)
    {
    this->ren->GetActors()->RemoveAllItems();
    if (this->GetDocument()->m_actor)
      {
      this->ren->AddActor(this->GetDocument()->m_actor);
      // force creation of a cmaera if one doesn't exist
      this->ren->GetActiveCamera();
      this->ren->ResetCamera();
      }
    }
  CView::OnUpdate(pSender, lHint, pHint);
}

void CVtkView::OnEditCopy() 
{
  if (OpenClipboard())
    {
    int *size = this->renWindow->GetSize();

    BeginWaitCursor();
    EmptyClipboard();
    SetClipboardData (CF_DIB, iren->GetDIB(size[0]*this->DIBScale,
					   size[1]*this->DIBScale,
					   this->DIBDepth));
    CloseClipboard();
    EndWaitCursor();
    }
}

/* old version
void CVtkView::OnEditCopy() 
{
  int *size = this->renWindow->GetSize();

  if (this->CopyMedium == NULL) 
    { // should only be created once
    CopyMedium = new STGMEDIUM;
    // using OLE structures for copying
    }
  CopySource = new COleDataSource;
  
  BeginWaitCursor();
  
  this->CopyMedium->tymed = TYMED_HGLOBAL;
  this->CopyMedium->hGlobal = this->iren->GetDIB(size[0]*this->DIBScale,size[1]*this->DIBScale,this->DIBDepth);
  this->CopySource->CacheData(CF_DIB,CopyMedium);
  this->CopySource->SetClipboard();	
  this->CopySource->FlushClipboard(); // give the jetzt source to the clipboard
  
  EndWaitCursor();
}
*/
